unit rOleTools;

{<
  rOleTool is set of OLE functions working with MSWord and converting
  files from one format to another.

  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Nov 2012)
  @lastmod(May 2016)
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Dialogs,
  Controls, ComObj;

function Word_OpenApp(ShowErrMsg: boolean): boolean;
procedure Word_CloseApp;
function Word_ConvertFileToPDF(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
function Word_ConvertFileToDOC(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
function Word_ConvertFileToDOCX(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
function Word_ConvertFileToODT(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
function Word_ConvertFileToRTF(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
function Word_ConvertFileExtension(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;


implementation

uses rstring, rtool, rdlg;

const
  // WdSaveFormat Enumeration (Word)
  // see http://msdn.microsoft.com/en-us/library/office/ff839952(v=office.15).aspx for full list

  wdDoNotSaveChanges = 0;
  wdFormatDocument = 0;
  wdFormatTemplate = 1;
  wdFormatText = 2;
  wdFormatTextLineBreaks = 3;
  wdFormatDOSText = 4;
  wdFormatDOSTextLineBreaks = 5;
  wdFormatRTF = 6;
  wdFormatUnicodeText = 7;
  wdFormatHTML = 8;
  wdFormatXMLDocument = 12;  //  MS OFFICE 2007+
  wdFormatXMLTemplate = 14;  //  MS OFFICE 2007+
  wdFormatDocumentDefault = 16;  //  MS OFFICE 2007+
  wdExportFormatPDF = 17;   //  MS OFFICE 2007+
  wdExportFormatXPS = 18;   //  MS OFFICE 2007+
  wdFormatOpenDocumentText = 23;  //  MS OFFICE 2010+
  wdFormatStrictOpenXMLDocument = 24; //  MS OFFICE 2013+
  wdExportOptimizeForOnScreen = 1;
  wdExportOptimizeForPrint = 0;

var
  Word, Doc: OleVariant;
  WordOpened: boolean;

function Word_OpenApp(ShowErrMsg: boolean): boolean;
begin
  Result:=false;
  try
    Word:=CreateOLEObject('Word.Application');
    WordOpened:=true;
    Result:=true;
  except
    if ShowErrMsg then
      ShowMessage(sMSWordNotFound);
  end;
end;

procedure Word_CloseApp;
begin
  if WordOpened then
    try
      WordOpened:=false;
      Word.Quit(wdDoNotSaveChanges);
    except
    end;
end;

function Word_IntConvertFile(InputFile, OutputFile: string; frm: OleVariant; ShowInfo, ShowErr: boolean): boolean;
var
  WasWordOpened: boolean;
begin
  Result:=false;
  try
    WasWordOpened:=WordOpened;
    if ShowInfo then
      rdlg.ShowInfo(Format(sFileGeneration,[ExtractFileName(OutputFile)]));
    try
      if not WordOpened then
        if not Word_OpenApp(ShowErr) then
          Exit;
      try
        Doc:=Word.Documents.Open(InputFile, false, true, false);
        if frm = wdExportFormatPDF then
          Doc.ExportAsFixedFormat(OutputFile, frm)
        else
          Doc.SaveAs(OutputFile, frm, EmptyParam, EmptyParam, false);
        Word.ActiveDocument.Close(false);
      finally
        if not WasWordOpened then
          Word_CloseApp;
      end;
    finally
      if ShowInfo then
        rdlg.ShowInfo('');
    end;
    Result:=true;
  except
    on E: Exception do
      if ShowErr then
        ShowMessage(sMSWordError + #13#10 + E.Message);
  end;
end;

function Word_ConvertFileToPDF(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
begin
  Result:=Word_IntConvertFile(InputFile, OutputFile, wdExportFormatPDF, ShowInfo, ShowErr);
end;

function Word_ConvertFileToDOC(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
begin
  Result:=Word_IntConvertFile(InputFile, OutputFile, wdFormatDocument, ShowInfo, ShowErr);
end;

function Word_ConvertFileToDOCX(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
begin
  Result:=Word_IntConvertFile(InputFile, OutputFile, wdFormatXMLDocument, ShowInfo, ShowErr);
end;

function Word_ConvertFileToODT(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
begin
  Result:=Word_IntConvertFile(InputFile, OutputFile, wdFormatOpenDocumentText, ShowInfo, ShowErr);
end;

function Word_ConvertFileToRTF(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
begin
  Result:=Word_IntConvertFile(InputFile, OutputFile, wdFormatRTF, ShowInfo, ShowErr);
end;

function Word_ConvertFileExtension(InputFile, OutputFile: string; ShowInfo: boolean = false; ShowErr: boolean = false): boolean;
var s: string;
begin
  Result:=false;
  s:=ExtractFileExt(OutputFile);
  if SameText(s,'.doc') then
    Result:=Word_ConvertFileToDOC(InputFile, OutputFile, ShowInfo, ShowErr);
  if SameText(s,'.docx') then
    Result:=Word_ConvertFileToDOCX(InputFile, OutputFile, ShowInfo, ShowErr);
  if SameText(s,'.odt') then
    Result:=Word_ConvertFileToODT(InputFile, OutputFile, ShowInfo, ShowErr);
  if SameText(s,'.pdf') then
    Result:=Word_ConvertFileToPDF(InputFile, OutputFile, ShowInfo, ShowErr);
  if SameText(s,'.rtf') then
    Result:=Word_ConvertFileToRTF(InputFile, OutputFile, ShowInfo, ShowErr);
end;

initialization

WordOpened:=false;

finalization

if WordOpened then
  Word_CloseApp;

end.
